@@ -4,6 +4,7 @@ Created on 15 Feb 2021
 @author: Christian Rab
 '''
 from abc import ABC,abstractmethod
+import decimal
 import math
 from timeit import default_timer as timer
 
@@ -45,7 +46,7 @@ class ReactionNetwork(ABC):
     '''
     self.non_species=['PE','PHOT','PHOTON','XPHOT','M','ELECTR','PHOTON',
                     'GRAIN','FORM','CRPHOT','CRP','SPONT','DESCR','DESPH','DESTH',
-                    'CRP','CP','e-','M',"dust"]
+                    'CRP','CP','e-','M',"dust","RADASS","PUMP"]
     '''
     A list of Species names that are considerd as non species (not really a chemical species) but
     might be included in the network.
@@ -96,6 +97,61 @@ class ReactionNetwork(ABC):
     # print("Time: ","{:4.2f}".format(timer()-starttime)+" s")
     return dups
 
+  def renameSpecies(self,old,new,variants=["","+","#"],log=True):
+    '''
+    Renames species with name `old` with species name `new` in the whole network.
+
+    Parameters
+    ----------
+
+    old : str
+      The species name of the species that should be renamed.
+
+    new : str
+      The new name of the species.
+
+    variants : array_like(str)
+      extension to the species names that should also be considered for renaming.
+      for example if `variants=["","+","#"] and the old name is SP and the new name is SPNEW
+      the routine would replace also species `SP,SP+,SP#` with `SPNEW,SPNEW+,SPNEW#` , respectively.
+      Types has to at least contain one element. Default:  `["","+","#"]`
+
+    log : boolean
+      log on changes on the screen. Default: True
+
+    Returns
+    -------
+    int :
+      The total number of changed Reactions.
+
+    '''
+    # number of changed done
+    nchanged=0
+    for reac in self.reactions:
+      for variant in variants:
+        spec=old+variant
+        nspecex=new+variant
+
+        # indices of the prodcuts the match the species
+        ip=[i for i,e in enumerate(reac.products) if e==spec]
+        ir=[i for i,e in enumerate(reac.reactants) if e==spec]
+        if len(ip)>0 or len(ir)>0:
+          if log:
+            print("Replace "+spec+" with "+nspecex+" in reaction ")
+            print(reac)
+          nchanged+=1
+
+        for i in ip:
+          reac.products[i]=nspecex
+
+        for i in ir:
+          reac.reactants[i]=nspecex
+
+    # reset internal species so that is is reloaded again in method `species
+    self._species=list()
+
+    return nchanged
+
   def compareSpecies(self,reactionNetwork):
     '''
     Compares the species list of two networks.
@@ -353,6 +409,10 @@ class Reaction(object):
     Any relevant comment for this particular reaction.
     """
 
+    # this is just for internal use to have coeff also as a decimal, to take into account the strange
+    # format of Reactions.in ... will only be filled in that case
+    self._coeff2=None
+
   def compare(self,reaction):
     '''
     Compares the Reaction to the passed reaction.
@@ -627,7 +687,7 @@ class ReactionNetworkPin(ReactionNetwork):
     '''
     return (eq["reactants"] and eq["products"]) and (eq["coeffs"]==False)
 
-  def load_reactions(self,filename="Reactions.in",fmt=None):
+  def load_reactions(self,filename="Reactions.in",fmt=None,threeReactants=True):
     '''
     Reads a reaction network in the format of the ProDiMo Reactions.in
 
@@ -717,8 +777,8 @@ class ReactionNetworkPin(ReactionNetwork):
   #           reac.products.append(sp)
 
         # very tricky, each species should take 8 characters, but the reading routine of PRoDiMo works
-        # also if one is only 7 charachters (don't know how), but it seems end the end the species can have
-        # mac 6 characters.
+        # also if one is only 7 charachters (don't know how), but it seems in the end the species can have
+        # max 7 characters.
         lensp=8
         idxnextsp=0
         for i in range(7):
@@ -736,6 +796,7 @@ class ReactionNetworkPin(ReactionNetwork):
           # if (idxnextsp+lensp)<61 and specs[idxnextsp+lensp]!=" ": idxnextsp-=1
 
         reac.coeffs[:]=[float(x.replace("D","E")) for x in ABC]
+        reac._coeff2=decimal.Decimal(ABC[1].strip())  # This is just for output and only for Reactions.in
         reac.comment=comm.strip()
         self.reactions.append(reac)
 
@@ -834,11 +895,37 @@ class ReactionNetworkPin(ReactionNetwork):
         # assume that it is always positiv
         fw.write("{:8.2E}".format(reac.coeffs[0]))
         fw.write(" ")
+
+        # the B coefficient
+        # Check for numbers that don't comply to the format
+        dig=str(reac._coeff2).split(".")
+        ndig=2
+        if (len(dig)>1):
+          ndig=len(dig[1])
         # can also be negative
-        fw.write("{:+5.2F}".format(reac.coeffs[1]))
+        if reac.coeffs[1]<0.0:
+          fmt="{:+5.2F}"
+        else:
+          fmt="{:5.2F}"
+        if ndig<3:
+          fw.write(fmt.format(reac.coeffs[1]))
+        else:
+          # print(reac,reac._coeff2)
+          fw.write(str(reac._coeff2))
+
         fw.write(" ")
+
+        # if the B coefficient has more digits than 5 we have a problem
+        # simple make the C coeff less long, like one would need to do if it is written by hand
+        lenstr2=len(str(reac._coeff2))
+        lenstr3=11
+        if lenstr2>5:
+          lenstr3=lenstr3-(lenstr2-5)
+          # print(lenstr3)
+
+        fmt="{:"+str(lenstr3).strip()+".4f}"
         # assume that it is always positiv
-        fw.write("{:12.4f}".format(reac.coeffs[2]))
+        fw.write(fmt.format(reac.coeffs[2]))
 
         fw.write("    ")
         fw.write(reac.comment)
