@@ -316,7 +316,7 @@ class PlotCasasim(object):
           if ccolors is None:
             ccolors="black"
           linecont=ax.contour(cube.data[velidx,:,:]*scalefac,np.array(clevels)*scalefac,origin="lower",
-                     linewidths=1.0,
+                     linewidths=0.5,
                      colors=ccolors)
 
         # ax.axis('equal')
@@ -497,9 +497,7 @@ class PlotCasasim(object):
     elif powerNormGamma is not None:
       norm=mcolors.PowerNorm(powerNormGamma,vmin=vmin,vmax=vmax)
 
-    im=ax.imshow(val,norm=norm,
-                   cmap=cmap,vmin=vmin,vmax=vmax,
-                   origin="lower")
+    im=ax.imshow(val,norm=norm,cmap=cmap,origin="lower")
 
     # FIXME: does not work with zlog
     if clevels is not None:
@@ -700,7 +698,7 @@ class PlotCasasim(object):
     CB.set_ticks(ticks)
     CB.set_label("velocity [km/s]")
 
-    self._closefig(fig)
+    return self._closefig(fig)
 
   def plot_pv(self,image,zlim=[None,None],ylim=[None,None],**kwargs):
     """
